
#include "rusinluk_XOR-bidex_BlockMng.h"
#include "rusinluk_XOR-bidex_XtPLA.h"

#include "rusinluk_XOR-bidex_utils.h"

#include <list>
#include <vector>
#include <string>
#include <sstream>
#include <cstring>
#include <cstdlib>
#include <ctime>
#include <iostream>
#include <iomanip>

#include "BOOM/Kernel/myExcept.h"
#include "BOOM/Kernel/mytime.h"
#include "BOOM/Kernel/bitvector.h"
#include "BOOM/Kernel/hashtable.h"
#include "BOOM/Kernel/Cube.h"
#include "BOOM/Kernel/Term.h"
#include "BOOM/Kernel/PLA_type.h"
#include "BOOM/Kernel/TTree.h"
#include "BOOM/Kernel/SoPLA.h"
#include "BOOM/Kernel/PLA.h"
#include "BOOM/Kernel/PLA_operations.h"
#include "BOOM/Kernel/utils.h"

#include "BOOM/Multilevel/Blif.h"

using namespace std;

#define ADRTH_DFT 3 // default threshold for address vars
#define ADRTH_MIN 1

#define L_GND "nodeGND"
#define L_VCC "nodeVCC"

#define APPEND(node) { blif->AppendPLA(node); delete node; }
#define TRY_APPEND(node,input) if (input == (int)node->inputs() - 1) { APPEND(node); }
	// if the last input is named

#define ARGS 8 // number of args

#define ARG_I 0
#define ARG_O 1
#define ARG_A 2
#define ARG_G 3
#define ARG_R 4
#define ARG_T 5
#define ARG_TO 6
#define ARG_S 7

char outFile_dft[] = "out.blif";

char
	*inFile = NULL,
	*outFile = outFile_dft;
int
	adrTh = ADRTH_DFT,
	seed;
float
	timeout;
bool
	dspRaw = false,
	dspGates = false,
	dspTim = false,
	enTimeout = false,
	enRandom = true;

PLA *pla = NULL;
vector<string>
	inL, outL;

Blif *blif = NULL;
unsigned long nn = 0; // node number

// just for testing
void displayBLIF(void) {
	list<SoPLA> *data = blif->getData();
	list<SoPLA>::iterator lsi = data->begin();
	unsigned int i, j;
	for (i = 1; i <= data->size(); i++) {
		cout << lsi->getName() << endl;
		for (j = 0; j < lsi->inputs(); j++) {
			cout << "  " << lsi->getIName(j) << endl;
		}
		lsi++;
	}
	cout << endl;
}

void displayAV(AdrV av) {
	if (av.val == ST_0) { cout << "NOT "; }
	cout << inL[av.id] << endl;
}

void space(int imm) {
	int i;
	for (i = imm; i > 0; i--) { cout << "  "; }
}

// output of a node
string nextName(void) {
	stringstream ss;
	ss << "node" << nn++;
	return ss.str();
}

// in input names
bool INamesContain(string name) {
	int i;
	for (i = inL.size() - 1; i >= 0; i--) {
		if (!inL[i].compare(name)) { return true; }
	}
	return false;
}

string getNextName(void) {
	string res;
	do { res = nextName(); } while (INamesContain(res));
	return res;
}

SoPLA *createINV(void) {
	SoPLA *node = new SoPLA("0");
	node->setName(getNextName());
	return node;
}

// just a prototype
SoPLA *createNode(void) {
	SoPLA *node = new SoPLA();
	node->setName(getNextName());
	return node;
}

void flushAVs(XtPLA *f, SoPLA *&parent, int &input, int &imm) {
	int
		nAV = f->getNAV(),
		i;
	if (dspGates) { space(imm); }
	if (f->getValF() == ST_0) {
		if (dspGates) { cout << "0" << endl; }
		parent->setIName(input, L_GND);
		TRY_APPEND(parent, input);
		return;
	}
	AdrV av;
	if (f->getValF() == ST_1) {
		switch (nAV) {
			case 0:
				if (dspGates) { cout << "1" << endl; }
				parent->setIName(input, L_VCC);
				TRY_APPEND(parent, input);
				return;
			case 1:
				av = f->getAVar();
				if (dspGates) { displayAV(av); }
				if (av.val == ST_0) {
					SoPLA *node = createINV();
					parent->setIName(input, node->getName());
					TRY_APPEND(parent, input);
					{ parent = node; input = 0; }
				}
				parent->setIName(input, inL[av.id]);
				TRY_APPEND(parent, input);
				return;
			default: break;
		}
	}
	if (!nAV) { return; }
	if (dspGates) { cout << "AND" << endl; imm++; }
	SoPLA *node = createNode();
	parent->setIName(input, node->getName());
	TRY_APPEND(parent, input);
	parent = node;
	int inputs = nAV;
	if (!f->isConstant()) { input = inputs++; }
	parent->setInputs(inputs);
	string map;
	for (i = 0; i < nAV; i++) {
		av = f->getAVar();
		if (dspGates) { space(imm); displayAV(av); }
		parent->setIName(i, inL[av.id]);
		map += ((av.val == ST_0)? '0' : '1');
	}
	if (!f->isConstant()) { map += '1'; }
	parent->AppendTerm(map);
	if (f->isConstant()) { APPEND(parent); }
}

SoPLA *createXOR(void) {
	SoPLA *node = new SoPLA(2);
	node->setName(getNextName());
	node->AppendTerm("01");
	node->AppendTerm("10");
	return node;
}

/*
also deletes the func
*/
void process(XtPLA *f, SoPLA *parent, int input, int imm) {
	f->simplify();
	if ((f->getNAV() >= adrTh) || f->isConstant()) { flushAVs(f, parent, input, imm); }
	if (f->isConstant()) { delete f; return; }
	if (dspGates) { space(imm); cout << "XOR" << endl; }
	SoPLA *node = createXOR();
	parent->setIName(input, node->getName());
	TRY_APPEND(parent, input);
	if (enRandom) { srand(time(NULL)); }
	XtPLA *f2 = f->step(
		rand() % f->getNV(),
		((rand() % 2)? ST_0 : ST_1)
	);
	process(f, node, 0, imm + 1);
	process(f2, node, 1, imm + 1);
}

bool findByOut(list<SoPLA>::iterator &lsi, string outName) {
	list<SoPLA> *data = blif->getData();
	lsi = data->begin();
	int i;
	for (i = data->size(); i > 0; i--) {
		if (!lsi->getName().compare(outName)) { return true; }
		lsi++;
	}
	return false;
}

void XORtoINV(SoPLA *node, int inputVCC) {
	string OName = node->getName();
	string IName = node->getIName(1 - inputVCC);
	node->clear();
	node->setInputs(1);
	node->AppendTerm("0");
	node->setName(OName);
	node->setIName(0, IName);
}

void nodeToConstant(SoPLA *node, bool vcc) {
	string OName = node->getName();
	node->clear();
	node->setInputs(1);
	node->setType(vcc);
	node->setName(OName);
	node->setIName(0, inL[0]); // any input
	node->AppendTerm("-");
}

void detach_vcc(void) {
	list<SoPLA> *data = blif->getData();
	list<SoPLA>::iterator lsi = data->begin();
	int i;
	for (i = data->size(); i > 0; i--) {
		if (lsi->inputs() != 2) { lsi++; continue; } // isn't XOR
		// maybe XOR now, but only XOR can have (one) vcc on input
		int j;
		for (j = 0; j <= 1; j++) {
			if (!lsi->getIName(j).compare(L_VCC)) { XORtoINV(&(*lsi), j); break; }
		}
		lsi++;
	}
}

void optimize(void) {
	detach_vcc();
	list<SoPLA>
		*data = blif->getData();
	list<SoPLA>::iterator
		ob, // output buffer
		lsi;
	unsigned int i;
	for (i = 0; i < pla->outputs(); i++) {
		findByOut(ob, outL[i]);
		string IName = ob->getIName(0);
		if (INamesContain(IName)) { continue; } // not a node
		if (!IName.compare(L_GND)) { nodeToConstant(&(*ob), false); continue; }
		if (!IName.compare(L_VCC)) { nodeToConstant(&(*ob), true); continue; }
		findByOut(lsi, IName);
		lsi->setName(outL[i]);
		data->erase(ob);
	}
}

void init(void) {
	pla = new PLA();
	pla->LoadPLA(inFile);
	pla->NameVars();
	inL = pla->getINames();
	outL = pla->getONames();
	blif = new Blif(inL, outL);
	SoPLA *node;
	{
		node = new SoPLA(1, false);
		node->setName(L_GND);
		node->setIName(0, inL[0]); // any input
		node->AppendTerm("-");
		APPEND(node);
	}
	{
		node = new SoPLA(1, true);
		node->setName(L_VCC);
		node->setIName(0, inL[0]); // any input
		node->AppendTerm("-");
		APPEND(node);
	}
}

void settingsInfo(void) {
	cout << "input file: " << inFile << endl;
	cout << "address vars threshold: " << adrTh << endl;
	cout << "output file: " << outFile << endl;
	{
		cout << "timeout: ";
		if (enTimeout)
			{ cout << timeout << " s"; }
		else
			{ cout << "--"; }
		cout << endl;
	}
	cout << endl;
}

void argsInfo(void) {
	cout << "arguments:" << endl;
	cout << "   INPUT_FILE            a file containing PLA" << endl;
	cout << "   [-o] OUTPUT_FILE      BLIF file" << endl;
	cout << "   [-a] NUMBER           address vars threshold" << endl;
	cout << "   [-g]                  display gates" << endl;
	cout << "   [-r]                  display raw data" << endl;
	cout << "   [-t]                  display timing only" << endl;
	cout << "   [-to] NUMBER          set timeout (in seconds, decimally)" << endl;
	cout << "   [-s] NUMBER           random seed" << endl;
}

bool getArgs(int argc, char *argv[]) {
	bool loaded[ARGS];
	int
		i, j;
	for (i = ARGS - 1; i >= 0; i--) {
		loaded[i] = false;
	}
	j = -1;
	for (i = 1; i < argc; i++) {
		if (j == -1) {
			j = ARG_I;
			if (!strcmp("-o", argv[i])) { j = ARG_O; }
			if (!strcmp("-a", argv[i])) { j = ARG_A; }
			if (!strcmp("-g", argv[i])) { j = ARG_G; }
			if (!strcmp("-r", argv[i])) { j = ARG_R; }
			if (!strcmp("-t", argv[i])) { j = ARG_T; }
			if (!strcmp("-to", argv[i])) { j = ARG_TO; }
			if (!strcmp("-s", argv[i])) { j = ARG_S; }
			if (loaded[j]) { return false; }
			loaded[j] = true;
			bool done = true;
			switch (j) {
				case ARG_I : inFile = argv[i]; break;
				case ARG_G : dspGates = true; break;
				case ARG_R : dspRaw = true; break;
				case ARG_T : dspTim = true; break;
				default : done = false; break;
			}
			if (done) { j = -1; }
			continue;
		}
		int a;
		switch (j) {
			case ARG_O : outFile = argv[i]; break;
			case ARG_A :
				a = atoi(argv[i]);
				if (a >= ADRTH_MIN) { adrTh = a; }
				break;
			case ARG_S :
				a = atoi(argv[i]);
				seed = a;
				enRandom = false;
				break;
			case ARG_TO :
				timeout = (float)atof(argv[i]);
				enTimeout = true;
				break;
			default : break;
		}
		j = -1;
	}
	return ((j == -1) && (inFile != NULL));
}

int main2(void) {
	init();
	if (!enRandom) { srand(seed); }
	BlockMng *bm = new BlockMng();
	XtPLA *f = NULL;
	if (!dspTim) { cout << "processing..." << endl; }
	float
		tim_load = 0, tim_decomp = 0, tim;
	int
		ok = 0;
	do {
		if (!pla->outputs()) { cout << "no outputs" << endl; ok = 1; break; }
		unsigned int i;
		for (i = 0; i < pla->outputs(); i++) {
			f = new XtPLA(bm);
			if (enTimeout) { f->setTimeout(timeout); }
			tim = GetMyCPUTime();
			f->load(pla, i);
			tim_load += GetMyCPUTime() - tim;
			if (dspRaw || dspGates) { cout << "for output: " << outL[i] << endl; }
			if (dspRaw) { cout << "raw data:" << endl; f->display(); }
			if (!f->isConsistent())
				{ cout << "PLA inconsistent: " << outL[i] << endl; ok = 1; break; }
			if (dspGates) { cout << "gates:" << endl; }
			SoPLA *ob = new SoPLA("1"); // output buffer
			ob->setName(outL[i]);
			tim = GetMyCPUTime();
			process(f, ob, 0, 0);
			tim_decomp += GetMyCPUTime() - tim;
			if (dspGates) { cout << endl; }
		}
		if (ok == 1) { break; }
		if (!dspTim) { cout << "optimizing..." << endl; }
		optimize();
		if (!dspTim) { cout << "saving BLIF..." << endl << endl; }
		blif->SaveBlif(outFile);
	}
		while (0);
	if (ok == 1) { delete f; }
	{ delete bm; delete pla; delete blif; }
	cout << tim_load << ";" << tim_decomp << ";";
	return ok;
}

int main(int argc, char *argv[]) {
	if (!getArgs(argc, argv)) { argsInfo(); return 1; }
	if (dspTim)
		{ dspRaw = false; dspGates = false; }
	else
		{ settingsInfo(); }
	int ok = 1;
	try {
		ok = main2();
	}
	catch (CException exc) {
		cout << exc.GetMsg() << endl;
	}
	return ok;
}

