
#include "rusinluk_XOR-bidex_XtPLA.h"

#include "rusinluk_XOR-bidex_Block.h"
#include "rusinluk_XOR-bidex_BlockMng.h"

#include "rusinluk_XOR-bidex_utils.h"

#include <list>
#include <vector>
#include <string>
#include <cstring>
#include <cstdlib>
#include <iostream>
#include <iomanip>

#include "BOOM/Kernel/myExcept.h"
#include "BOOM/Kernel/mytime.h"
#include "BOOM/Kernel/bitvector.h"
#include "BOOM/Kernel/hashtable.h"
#include "BOOM/Kernel/Cube.h"
#include "BOOM/Kernel/Term.h"
#include "BOOM/Kernel/PLA_type.h"
#include "BOOM/Kernel/TTree.h"
#include "BOOM/Kernel/SoPLA.h"
#include "BOOM/Kernel/PLA.h"
#include "BOOM/Kernel/PLA_operations.h"
#include "BOOM/Kernel/utils.h"

using namespace std;

XtPLA::XtPLA(BlockMng *bm) {
	this->bm = bm;
	data[0] = NULL;
	data[1] = NULL;
	adrV = NULL;
	idV = NULL;
	enTimeout = false;
}

XtPLA::XtPLA(XtPLA *a) {
	if (a == this) { return; }
	memcpy(this, a, sizeof(XtPLA));
	{ adrV = new AdrV[nAV + nv]; memcpy(adrV, a->adrV, sizeof(AdrV) * nAV); }
	{ idV = new int[nv]; memcpy(idV, a->idV, sizeof(int) * nv); }
	int i;
	for (i = 1; i >= 0; i--) {
		Block
			*b = bm->getBlock(),
			*b0 = a->data[i];
		data[i] = b;
		while (1) {
			memcpy(b->data, b0->data, sizeof(char) * BLOCK_SIZE);
			if ((b0 = b0->next) == NULL) { break; }
			b = b->next = bm->getBlock();
		}
	}
}

XtPLA::~XtPLA(void) {
	rem();
}

void XtPLA::rem(void) {
	bm->relBlocks(data);
	bm->relBlocks(data + 1);
	delete[] adrV;
	delete[] idV;
}

bool XtPLA::isConsistent(void) {
	if (isConstant()) { return true; }
	int
		i, j;
	char
		*c1, *c2;
	toBegin(0, 0);
	for (i = nc[0]; i > 0; i--) {
		c1 = getNextCube(0);
		toBegin(1, 1);
		for (j = nc[1]; j > 0; j--) {
			c2 = getNextCube(1);
			if (common(c1, c2) != -1) { return false; }
		}
	}
	return true;
}

int XtPLA::getCubeSize(char *cube) {
	int
		i,
		res = 0;
	for (i = nv - 1; i >= 0; i--) {
		if ((cube[i] & ST_X) == ST_X) { res++; }
	}
	return res;
}

int XtPLA::common(char *c1, char *c2) {
	char a;
	int
		i,
		res = 0;
	for (i = nv - 1; i >= 0; i--) {
		a = (c1[i] & c2[i]);
		if (a & XtPLA_IGNVAR) { continue; }
		a &= ST_X; // removing signs
		if (a == ST_N) { return -1; } // val collision => no common area
		if (a == ST_X) { res++; }
	}
	return res;
}

int XtPLA::joinable(char *c1, char *c2) {
	int
		i,
		res = -1;
	char
		d1, d2; // data
	for (i = nv - 1; i >= 0; i--) {
		d1 = (c1[i] & ST_X);
		d2 = (c2[i] & ST_X);
		if (d1 == d2) { continue; }
		if ((d1 == ST_X) || (d2 == ST_X)) { return -1; }
		// here (0,1) or (1,0)
		if (res == -1)
			{ res = i; } // first collision
		else
			{ return -1; } // another collision
	}
	return res;
}

void XtPLA::remRedundantCubes(void) {
	int
		i, j, k,
		a;
	char
		*c1, *c2;
	bool
		joined;
	for (i = 1; i >= 0; i--) {
		do {
			if (enTimeout) { checkTimeout(); }
			joined = false;
			toBegin(0, i);
			for (j = nc[i]; j > 0; j--) {
				c1 = getNextCube(0);
				iter[1] = iter[0];
				for (k = j - 1; k > 0; k--) {
					c2 = getNextCube(1);
					a = common(c1, c2);
					if (a == getCubeSize(c2))
						{ *c2 = XtPLA_NOCUBE; nc[i]--; j--; continue; }
					if (a == getCubeSize(c1)) { *c1 = XtPLA_NOCUBE; nc[i]--; break; }
					a = joinable(c1, c2);
					if (a != -1) {
						c1[a] = ST_X;
						{ *c2 = XtPLA_NOCUBE; nc[i]--; j--; }
						joined = true;
					}
				}
			}
		}
			while (joined);
	}
}

void XtPLA::simplify(void) {
	if (checkConstant()) { return; }
	tryRemVars();
	if (checkConstant()) { return; }
	restruct();
	remRedundantCubes();
	restruct();
}

/*
assumptions:
 - the iterator must be at the position of the last cube
     - calling toEnd()
*/
void XtPLA::addCube(char *cube, int itr) {
	IterC *a = iter + itr;
	if (a->i == (cib - 1)) {
		a->b = a->b->next = bm->getBlock();
		a->i = 0;
	}
	else {
		a->i++;
	}
	memcpy(a->b->data + a->i * ov, cube, sizeof(char) * nv);
}

void XtPLA::editVar(int iV, bool sign) {
	char *cube;
	int
		i, j;
	for (i = 1; i >= 0; i--) {
		toBegin(0, i);
		for (j = nc[i]; j > 0; j--) {
			cube = getNextCube(0);
			if (sign)
				{ cube[iV] |= XtPLA_IGNVAR; }
			else
				{ cube[iV] &= ~XtPLA_IGNVAR; }
		}
	}
}

void XtPLA::remVar(int iV) {
	char *cube;
	int
		i, j;
	idV[iV] = idV[nv - 1];
	for (i = 1; i >= 0; i--) {
		toBegin(0, i);
		for (j = nc[i]; j > 0; j--) {
			cube = getNextCube(0);
			cube[iV] = cube[nv - 1];
		}
	}
	nv--;
}

void XtPLA::remVarAsAdr(int iV, char valV) {
	{ adrV[nAV].id = idV[iV]; adrV[nAV].val = valV; nAV++; }
	if (nv == 1) { nc[0] = 0; return; }
	stepBySym(iV, valV, false); // removing collisions (1-cubes priority)
}

void XtPLA::tryRemVars(void) {
	int
		i, j;
	char
		*c, val;
	for (i = nv - 1; i >= 0; i--) {
		editVar(i, true);
		if (!isConsistent()) { editVar(i, false); continue; }
		remVar(i);
	}
	for (i = nv - 1; i >= 0; i--) {
		val = ST_X;
		toBegin(0, 1);
		for (j = nc[1]; j > 0; j--) {
			c = getNextCube(0);
			if ((val &= ~c[i]) == ST_N) { break; }
		}
			/*
			test for address var
			placement of 1-cubes (val is inverted)
			*/
		if (val != ST_N) { remVarAsAdr(i, ~val & ST_X); }
	}
}

bool XtPLA::checkConstant(void) {
	valF = ST_X;
	if (!nc[0]) { valF = ST_1; }
	if (!nc[1]) { valF = ST_0; }
	if (isConstant()) {
		int i;
		for (i = 1; i >= 0; i--) { bm->relBlocks(data + i); }
		return true;
	}
	return false;
}

bool XtPLA::isConstant(void) {
	return (valF != ST_X);
}

bool XtPLA::addible(char *cube, int val) {
	int
		size = getCubeSize(cube),
		i,
		n = 1;
	bool
		res = true;
	IterC
		bak = iter[0];
	toBegin(0, val);
	for (i = nc[val]; i > 0; i--) {
		if (
			(common(cube, getNextCube(0)) == size) && // subset?
			!(n--) // one subset = the same cube (deleted later)
		)
			{ res = false; break; }
	}
	iter[0] = bak;
	return res;
}

/*
generates and adds all the cubes from the target cube which haven't any common area
  with the source cube
assumptions:
 - the iterator must be at the position of the last cube
returns number of added cubes
*/
void XtPLA::cubeDecomp(char *src, char *tg, int itr, int val, bool sign) {
	int i;
	for (i = nv - 1; i >= 0; i--) {
		if (((src[i] & ST_X) != ST_X) && ((tg[i] & ST_X) == ST_X)) {
			tg[i] = (~src[i] & ST_X); // removing collision
			if (addible(tg, val)) {
				if (sign) { *tg |= XtPLA_NEWCUBE; }
				{ addCube(tg, itr); nc[val]++; }
			}
			tg[i] = ST_X;
		}
	}
}

void XtPLA::stepBySym(int iV, char valV, bool inv) {
	int
		i, j;
	char
		*c, *c2;
	toEnd(0, 0); // for adding inverted 1-cubes + pieces of 0-cubes
	if (inv) {
		toBegin(1, 1);
		for (i = nc[1]; i > 0; i--) {
			c = getNextCube(1);
			if ((c[iV] & valV) == ST_N) {
				{ addCube(c, 0); nc[0]++; }
				{ *c = XtPLA_NOCUBE; nc[1]--; }
			}
		}
			// inverting all 1-cubes of the opposite val
	}
	toBegin(1, 1);
	for (i = nc[1]; i > 0; i--) {
		if (enTimeout) { checkTimeout(); }
		c = getNextCube(1);
		c[iV] = (~valV & ST_X); // remapping the area
		toBegin(2, 0);
		for (j = nc[0]; j > 0; j--) {
			c2 = getNextCube(2);
			if (
				((c2[iV] & valV) == ST_N) && // val/dual 0-cubes don't collide
				(common(c, c2) != -1)
			) {
				cubeDecomp(c, c2, 0, 0, false);
				{ *c2 = XtPLA_NOCUBE; nc[0]--; }
			}
		}
	}
	remVar(iV);
}

void XtPLA::stepByAdr(int iV, char valV) {
	int
		i, j, k, i2;
	char
		*c, *c2,
		*mask = new char[nv], // where will be inversions
		*cube = new char[nv]; // backup
	bool
		dual, dual2, origCube;
	toEnd(0, 0); // for adding pieces of 0-cubes
	toEnd(1, 1); // for adding pieces of 1-cubes
	toBegin(2, 1);
	for (i = nc[1]; i > 0; i--) {
		if (enTimeout) { checkTimeout(); }
		c = getNextCube(2);
		if ((c[iV] & valV) == ST_N) { *c |= XtPLA_NEWCUBE; continue; } // a trick
		memcpy(mask, c, sizeof(char) * nv);
		dual = (mask[iV] == ST_X);
		mask[iV] = (~valV & ST_X); // remapping the area
		if ((c[iV] &= ~valV) == ST_N)
			{ *c = XtPLA_NOCUBE; nc[1]--; } // removing the val side
		else
			{ *c |= XtPLA_NEWCUBE; }
		for (j = 1; j >= 0; j--) {
			if ((j == 0) && dual) { continue; }
			toBegin(3, j);
			for (k = nc[j]; k > 0; k--) {
				c2 = getNextCube(3);
				dual2 = ((c2[iV] & ST_X) == ST_X);
				if (
					(*c2 & XtPLA_IGNCUBE) ||
					((c2[iV] & ST_X) == valV) ||
					((j == 0) && dual2) ||
						// no collisions/common area if true
					(common(mask, c2) == -1)
				)
					{ continue; }
				origCube = !(*c2 & XtPLA_NEWCUBE);
				*c2 &= ST_X;
				if (dual2) {
					memcpy(cube, c2, sizeof(char) * nv);
					cube[iV] = valV;
					c2[iV] = (~valV & ST_X);
				}
				cubeDecomp(mask, c2, j, j, true);
				for (i2 = nv - 1; i2 >= 0; i2--) { c2[i2] &= mask[i2]; }
				*c2 |= XtPLA_IGNCUBE;
				{ addCube(c2, 1 - j); nc[1 - j]++; }
				{ *c2 = XtPLA_NOCUBE; nc[j]--; }
				if (dual2) {
					{ memcpy(c2, cube, sizeof(char) * nv); nc[1]++; }
				}
				else {
					if ((j == 1) && origCube) { i--; } // decrement of expected count
				}
			}
		}
	}
	for (i = 1; i >= 0; i--) {
		toBegin(2, i);
		for (j = nc[i]; j > 0; j--) {
			*getNextCube(2) &= ST_X;
		}
	}
		// removing signs
	remVarAsAdr(iV, ~valV & ST_X);
	delete[] mask;
	delete[] cube;
}

XtPLA *XtPLA::step(int iV, char valV) {
	if (
		(nv < 2) || (iV < 0) || (iV >= nv) ||
		((valV != ST_0) && (valV != ST_1)) ||
		isConstant()
	)
		{ return NULL; }
	XtPLA *a = new XtPLA(this);
	stepBySym(iV, valV, true);
	a->stepByAdr(iV, valV);
	return a;
}

AdrV XtPLA::getAVar(void) {
	if (nAV < 0) { return adrV[0]; } // return something
	return adrV[--nAV];
}

int XtPLA::getNV(void) {
	return nv;
}

int XtPLA::getNAV(void) {
	return nAV;
}

char XtPLA::getValF(void) {
	return valF;
}

char *XtPLA::getNextCube(int itr) {
	IterC *a = iter + itr;
	char *res;
	while (1) {
		if (a->i >= cib) { a->b = a->b->next; a->i = 0; }
		res = (a->b->data + ov * a->i++);
		if (*res != XtPLA_NOCUBE) { return res; }
	}
}

void XtPLA::toBegin(int itr, int val) {
	iter[itr].b = data[val];
	iter[itr].i = 0;
}

void XtPLA::toEnd(int itr, int val) {
	toBegin(itr, val);
	IterC *a = iter + itr;
	int n = nc[val];
	char *cube;
	while (1) {
		if (a->i >= cib) { a->b = a->b->next; a->i = 0; }
		cube = (a->b->data + ov * a->i);
		if ((*cube != XtPLA_NOCUBE) && !(--n)) { return; }
		a->i++;
	}
}

void XtPLA::restruct(void) {
	if (isConstant()) { return; }
	char *cube;
	int
		i, j,
		cibNew = BLOCK_SIZE / nv,
		iNew; // index of cube
	Block
		*bNew,
		*a;
	for (i = 1; i >= 0; i--) {
		a = bNew = bm->getBlock();
		iNew = 0;
		toBegin(0, i);
		for (j = nc[i]; j > 0; j--) {
			cube = getNextCube(0);
			if (iNew == cibNew) {
				a = a->next = bm->getBlock();
				iNew = 0;
			}
			memcpy(a->data + nv * iNew++, cube, sizeof(char) * nv);
		}
		bm->relBlocks(data + i);
		data[i] = bNew;
	}
	cib = cibNew;
	ov = nv;
}

void XtPLA::load(PLA *pla, int iO) {
	rem();
	char *cube = NULL;
	SoPLA *sopla = NULL;
	{ nv = (int)pla->inputs(); ov = nv; }
	cib = BLOCK_SIZE / nv;
	{ adrV = new AdrV[nv]; nAV = 0; }
	idV = new int[nv];
	int i;
	for (i = nv - 1; i >= 0; i--) {
		idV[i] = i; // primary id set
	}
	for (i = 1; i >= 0; i--) {
		iter[i].b = data[i] = bm->getBlock();
		iter[i].i = -1;
		nc[i] = 0;
	}
		// in this case, iterators are binded with cube vals
	cube = new char[nv];
	for (i = 1; i >= 0; i--) {
		switch (i) {
			case 0: sopla = pla->getOffSet(iO); break;
			case 1: sopla = pla->getOnSet(iO); break;
			default: break;
		}
		list<Cube> *lc = sopla->getData();
		list<Cube>::iterator lci = lc->begin();
		int j;
		for (j = lc->size(); j > 0; j--) {
			string str = (lci++)->getCube();
			int k;
			for (k = nv - 1; k >= 0; k--) {
				switch (str[k]) {
					case '0': cube[k] = ST_0; break;
					case '1': cube[k] = ST_1; break;
					default: cube[k] = ST_X; break;
				}
			}
			{ addCube(cube, i); nc[i]++; }
		}
	}
	checkConstant();
	delete[] cube;
}

void XtPLA::displayCube(char *cube) {
	int i;
	for (i = 0; i < nv; i++) {
		switch (cube[i]) {
			case ST_0 : cout << "0"; break;
			case ST_1 : cout << "1"; break;
			case ST_X : cout << "-"; break;
			default : cout << "(" << ((unsigned int)cube[i] & 0xff) << ")"; break;
		}
	}
}

void XtPLA::display(void) {
	if (isConstant()) {
		cout << "func is constant: " << ((valF == ST_0)? "0" : "1") << endl;
		return;
	}
	int
		i, j;
	IterC
		bak = iter[0];
	for (i = 0; i <= 1; i++) {
		toBegin(0, i);
		for (j = nc[i]; j > 0; j--) {
			displayCube(getNextCube(0));
			cout << " " << i << endl;
		}
		cout << endl;
	}
	iter[0] = bak;
}

void XtPLA::setTimeout(float to) {
	if (to > 0) { timeout = to; enTimeout = true; }
}

void XtPLA::unsetTimeout(void) {
	enTimeout = false;
}

void XtPLA::checkTimeout(void) {
	if (GetMyCPUTime() > timeout) { exit(1); }
}

