
#ifndef rusinluk_XOR_bidex_XtPLA_h
#define rusinluk_XOR_bidex_XtPLA_h

#include "rusinluk_XOR-bidex_Block.h"
#include "rusinluk_XOR-bidex_BlockMng.h"

#include "rusinluk_XOR-bidex_utils.h"

#include <list>
#include <vector>
#include <string>

#include "BOOM/Kernel/myExcept.h"
#include "BOOM/Kernel/mytime.h"
#include "BOOM/Kernel/bitvector.h"
#include "BOOM/Kernel/hashtable.h"
#include "BOOM/Kernel/Cube.h"
#include "BOOM/Kernel/Term.h"
#include "BOOM/Kernel/PLA_type.h"
#include "BOOM/Kernel/TTree.h"
#include "BOOM/Kernel/SoPLA.h"
#include "BOOM/Kernel/PLA.h"
#include "BOOM/Kernel/PLA_operations.h"
#include "BOOM/Kernel/utils.h"

using namespace std;

#define XtPLA_IGNVAR (char)0x04 // ignore var
#define XtPLA_IGNCUBE (char)0x08 // ignore cube
#define XtPLA_NEWCUBE (char)0x10
#define XtPLA_NOCUBE (char)0x80 // cube deleted, non-returnable
	// signs

/*
address var
*/
typedef struct AdrV {
	int id;
	char val;
} AdrV;

/*
cube iterator
*/
typedef struct IterC {
	Block *b;
	int i; // index of char in local block
} IterC;

/*
extended PLA
cubes are stored in an array
there are also "address" variables here which are not contained in PLA
	- predefined values, function is zero outside them
	- eliminating by AND-tree
*/
class XtPLA {
	private:
		BlockMng
			*bm; // used block manager
		Block
			*data[2]; // data of func results (cubes = restrictions for 0 and 1)
		IterC
			iter[4]; // iterators
		AdrV
			*adrV; // address vars
		char
			valF; // if the func is constant
		int
			nc[2], // numbers of cubes
			nAV, // number of address vars
			*idV, // ids of vars
			nv, // number of vars
			ov, // original number of vars
			cib; // maximum cubes in block
		float
			timeout;
		bool
			enTimeout;
	public:
		XtPLA(BlockMng *);
		XtPLA(XtPLA *);
		~XtPLA(void);
		bool isConsistent(void); // searches for collisions
		void simplify(void);
			/* removes symmetric vars
			 * adds address vars
			 * removes redundant cubes
			 */
		bool isConstant(void);
		XtPLA *step(int iV, char valV);
			/* XOR-bidecomposition by var
			 * returns the second argument
			 */
		AdrV getAVar(void); // removes and returns the last address var
		int getNV(void);
		int getNAV(void);
		char getValF(void);
		void load(PLA *pla, int iO);
		void display(void); // displays all the cubes, test
		void setTimeout(float to);
		void unsetTimeout(void);
	private:
		void rem(void); // releases memory
		int getCubeSize(char *cube); // number of DCs
		int common(char *c1, char *c2);
			/* the exponent of cubes' common states
			 *   - always a power of 2, returns -1 if zero
			 */
		int joinable(char *c1, char *c2); // index of join if joinable
		void remRedundantCubes(void); // rem cube duplicities or subsets
		void addCube(char *cube, int itr); // adds a cube to the end of data
		void editVar(int iV, bool sign); // sets/unsets the ignore sign to var
		void remVar(int iV); // removes var, without collision checking
		void remVarAsAdr(int iV, char valV);
			// removes var and adds it to the address var list
		void tryRemVars(void); // complex utility to reducing vars
		bool checkConstant(void); // checks and sets the func to constant
		bool addible(char *cube, int val); // whether no subset of any cube of the val
		void cubeDecomp(char *src, char *tg, int itr, int val, bool sign);
			// generates the minimal count of cubes which don't collide with src
		void stepBySym(int iV, char valV, bool inv);
			/* selects from func the part addressed by var's val
			 * makes all the 1-cubes of the part symmetric
			 * inv = inverting 1-cubes of the opposite val
			 */
		void stepByAdr(int iV, char valV);
			/* provides inversions by 1-cubes of the part in the second part
			 * clears the part of 1-cubes
			 */
		char *getNextCube(int itr);
		void toBegin(int itr, int val); // takes the selected iterator to begin of data
		void toEnd(int itr, int val); // to the last cube
		void restruct(void); // data restructuring
		void displayCube(char *cube); // displays one cube, test
		void checkTimeout(void);
};

#endif

